package com.dali.galery

import android.graphics.drawable.Drawable
import android.view.View

/**
 * Strategies for displaying images on different View elements
 *
 * Example:
 * class MyViewStrategy: ViewStrategy {
 *
 *      override fun onDraw(view: View, drawable: Drawable?) {
 *          when (view) {
 *              is ImageView -> view.setImageDrawable(drawable)
 *              is ViewGroup -> view.background = drawable
 *          }
 *      }
 * }
 *
 */
interface ViewStrategy {

    /**
     * The method overrides the use of the image on different elements in the Views
     * @param view View element
     * @param drawable image
     */
    fun onDraw(view: View, drawable: Drawable?)
}