package com.dali.galery.network

import android.content.Context
import android.graphics.drawable.Drawable
import android.util.Log
import com.dali.galery.reflection.DaliClient
import okhttp3.OkHttpClient
import okhttp3.Request
import okhttp3.Response
import java.io.IOException
import java.util.concurrent.CountDownLatch
import java.util.concurrent.Executors

/**
 * API Image loading and caching interactions
 */
internal class DaliApi {

    private val fileManager = FileManager()
    private val downloadService = Executors.newCachedThreadPool()
    private val client: OkHttpClient = DaliClient.get().okHttpClient ?: OkHttpClient()
    private val requestBuilder: Request.Builder = DaliClient.get().okHttpRequest ?: Request.Builder()

    /**
     *  Check existing file and upload image from file.
     */
    internal fun loadImage(url: String, context: Context, name: String): Result<Drawable?> {
        if (fileManager.checkExistFile(context, name)) {
            return fileManager.loadImageFromFile(context, name)
        }
        var result: Result<Drawable?>? = null
        val fullUrl = DaliClient.get().baseUrl + DaliClient.get().getImageScopeSettingByName(name).uri + url
        if (fullUrl.contains("http", true).not()) {
            Log.e("Dali.BadUrlException", "Url is $fullUrl")
            return Result.Error(Exception("Bad url"))
        }
        val request =
            requestBuilder.url(fullUrl).build()
        val latch = CountDownLatch(1)
        downloadService.execute {
            try {
                val response: Response = client.newCall(request).execute()
                when (response.code) {
                    in DaliClient.get().responseStatusCodes -> fileManager.saveImageToFile(
                        context,
                        response.body?.byteStream(),
                        name
                    )
                    else -> result = Result.Error(Exception("File load error, code " + response.code))
                }
                latch.countDown()
            } catch (e: IOException) {
                latch.countDown()
                result = Result.Error(Exception("File cast error", e))
            }
        }
        try {
            latch.await()
        } catch (e: InterruptedException) {
            e.printStackTrace()
        }
        return result ?: fileManager.loadImageFromFile(context, name)
    }

    internal fun removeCacheFile(context: Context, name: String?) {
        if (name != null) fileManager.removeFile(context, name)
    }
}