package com.dali.galery.network

import android.content.Context
import android.graphics.drawable.BitmapDrawable
import android.graphics.drawable.Drawable
import android.util.Log
import com.dali.galery.reflection.DaliClient
import java.io.File
import java.io.FileInputStream
import java.io.FileOutputStream
import java.io.IOException
import java.io.InputStream

class FileManager {

    /**
     *  Download an image from the file.
     */
    internal fun loadImageFromFile(context: Context, name: String): Result<Drawable?> {
        if (checkExistFile(context, name)) {
            var drawable: BitmapDrawable? = null
            val inputStream: FileInputStream
            try {
                val file = File(getFile(name, context), name)
                inputStream = FileInputStream(file)
                drawable = BitmapDrawable(context.resources, inputStream)
                inputStream.close()
            } catch (e: Exception) {
                e.printStackTrace()
            }
            // Удалить файл и показать заглушку, если файл не получается открыть
            if (drawable?.bitmap == null) {
                removeFile(context, name)
                return Result.Error(Exception("File not found error"))
            }
            return Result.Success(drawable, name)
        }
        return Result.Error(Exception("File not found error"))
    }

    /**
     * Save an image from the file.
     */
    internal fun saveImageToFile(context: Context, stream: InputStream?, fileName: String) {
        if (stream != null) {
            val file = File(getFile(fileName, context), "$fileName$TEMP")
            copyInputStreamToFile(stream, file)
        }
        if (checkExistFile(context, "$fileName$TEMP")) {
            rename(context, fileName)
        }
    }

    /**
     * Remove file
     */
    internal fun removeFile(context: Context, name: String) {
        if (checkExistFile(context, name)) {
            val file = File(getFile(name, context), name)
            file.delete()
        }
    }

    /**
     * File availability check
     */
    internal fun checkExistFile(context: Context, name: String): Boolean {
        val file = File(getFile(name, context), name)
        return file.exists() && !file.isDirectory
    }

    @Throws(IOException::class)
    private fun copyInputStreamToFile(inputStream: InputStream, file: File) {
        FileOutputStream(file, false).use { outputStream ->
            var read: Int
            val bytes = ByteArray(DEFAULT_BUFFER_SIZE)
            while (inputStream.read(bytes).also { read = it } != -1) {
                outputStream.write(bytes, 0, read)
            }
        }
    }

    private fun rename(context: Context, fileName: String) {
        val file = File(getFile(fileName, context), "$fileName$TEMP")
        val file2 = File(getFile(fileName, context), fileName)
        val success = file.renameTo(file2)
        if (!success) {
            Log.i("DaliLoad", "Upload Image: Success")
        }
    }

    private fun getFile(scopeName: String, context: Context): File {
        var fileDir: File? = null
        val dir = DaliClient.get().getImageScopeSettingByName(scopeName).dirName
        if (dir.isNotEmpty()) {
            fileDir = context.getDir(dir, Context.MODE_PRIVATE)
        }
        return fileDir ?: context.filesDir
    }

    companion object {
        private const val TEMP = ".temp"
    }
}