package com.dali.galery.reflection

import android.app.Activity
import android.content.Context
import android.content.ContextWrapper
import android.view.LayoutInflater
import com.dali.galery.reflection.viewcategory.ViewDaliActivityFactory
import com.dali.galery.reflection.viewcategory.ViewDaliLayoutInflater
import kotlin.LazyThreadSafetyMode.NONE

/**
 * Uses the default configuration from [DaliClient]
 *
 * Remember if you are defining default in the
 * [DaliClient] make sure this is initialised before
 * the activity is created.
 *
 * @param base ContextBase to Wrap
 */
class ViewDaliContextWrapper private constructor(base: Context) : ContextWrapper(base) {

    private val inflater: ViewDaliLayoutInflater by lazy(NONE) {
        ViewDaliLayoutInflater(
            LayoutInflater.from(baseContext), this, false
        )
    }

    override fun getSystemService(name: String): Any? {
        if (Context.LAYOUT_INFLATER_SERVICE == name) {
            return inflater
        }
        return super.getSystemService(name)
    }

    companion object {

        /**
         * Uses the default configuration from [DaliClient]
         *
         * Remember if you are defining default in the [DaliClient] make sure this
         * is initialised before the activity is created.
         *
         * @param base ContextBase to Wrap.
         * @return ContextWrapper to pass back to the activity.
         */
        @JvmStatic
        fun wrap(base: Context): ContextWrapper {
            return ViewDaliContextWrapper(base)
        }

        /**
         * Get the ViewDali Activity Fragment Instance to allow callbacks for when views are created.
         *
         * @param activity The activity the original that the ContextWrapper was attached too.
         * @return Interface allowing you to call onActivityViewCreated
         */
        @JvmStatic
        internal fun get(activity: Activity): ViewDaliActivityFactory {
            if (activity.layoutInflater !is ViewDaliLayoutInflater) {
                throw RuntimeException(
                    "This activity does not wrap the Base Context! See ViewDaliContextWrapper.wrap(Context)"
                )
            }
            return activity.layoutInflater as ViewDaliActivityFactory
        }
    }
}
