package com.dali.galery.reflection.interceptor

import android.content.res.TypedArray
import android.view.View
import com.dali.galery.Dali
import com.dali.galery.network.DaliOrchestrator
import com.dali.galery.reflection.inflate.data.InflateResult
import com.dali.galery.utils.PathFormator
import com.dali.galery.utils.PathKeeper
import com.dali.android.processor.ResourceProperties
import com.dali.android.processor.kspPackage
import com.dali.galery.R
import com.dali.galery.reflection.DaliClient

/**
 * [View] interceptor to get tag field and display images
 * Initiates the download process
 */
internal class InterceptView : Interceptor {

    private val daliOrchestrator = DaliOrchestrator.instance
    private val dali = Dali.getInstance()

    override fun intercept(chain: Interceptor.Chain): InflateResult {
        val result = chain.proceed(chain.request())
        val typedArray: TypedArray = result.context.obtainStyledAttributes(result.attrs, intArrayOf(R.attr.resourceProperties))
        try {
            val tag: CharSequence? = typedArray.getText(0)
            if (!tag.isNullOrEmpty()) {
                val view = result.view
                applyImage(view, getResourceProperties(tag))
            }
        } catch (e: Exception) {
            e.printStackTrace()
        } finally {
            typedArray.recycle()
        }
        return result
    }

    private fun applyImage(view: View?, resourceProperties: ResourceProperties) {
        if (dali.addView(resourceProperties, view)) {
            daliOrchestrator.update(resourceProperties, view?.context?.applicationContext, DaliClient.get().callback, DaliClient.get().transformer)
        }
    }

    private fun getResourceProperties(tag: CharSequence): ResourceProperties {

        val pathKeeper: PathKeeper = PathFormator.getName(tag, kspPackage())

        return try {
            val clazz = Class.forName(pathKeeper.getClassNamePath())
            val field = clazz.getDeclaredField(pathKeeper.fieldName)
            field.isAccessible = true
            field.get(clazz) as ResourceProperties
        } catch (e: NoSuchFieldException) {
            throw RuntimeException("Error tag $tag resource not found", e)
        } catch (e: ClassNotFoundException) {
            throw RuntimeException("Error tag $tag resource not found", e)
        }
    }
}