package com.dali.galery.utils

import android.view.View
import android.view.animation.Animation

/**
 * Utility class for animation
 */
class DaliAnimation {

    companion object {

        /**
         * Image Replacement Animation
         * Replacing an image through animation
         * @param view View element
         * @param draw image
         * @param animOut animation Out
         * @param animIn animation In
         */
        @JvmStatic
        fun <T : View> imageViewAnimatedChange(view: T, draw: Draw<T>, animOut: Animation, animIn: Animation) {
            animOut.setAnimationListener(object : Animation.AnimationListener {
                override fun onAnimationStart(animation: Animation) {}
                override fun onAnimationRepeat(animation: Animation) {}
                override fun onAnimationEnd(animation: Animation) {
                    draw.onDraw(view)
                    animIn.setAnimationListener(object : Animation.AnimationListener {
                        override fun onAnimationStart(animation: Animation) {}
                        override fun onAnimationRepeat(animation: Animation) {}
                        override fun onAnimationEnd(animation: Animation) {}
                    })
                    view.startAnimation(animIn)
                }
            })
            view.startAnimation(animOut)
        }
    }

    interface Draw<T> {

        /**
         * The method overrides the use of the image on different elements in the Views
         * @param view View element
         */
        fun onDraw(view: T)
    }
}